//#define _DEBUG
/* ========================================
 *
 * PSoC5-EVB テストプログラム
 * UART Baudrate : 115.2kbps
 *
 * つくったひと : cronos_sv
 *
 * ========================================
*/
#include <stdlib.h>
#include <string.h>
#include <device.h>
#include "main.h"
#include "sdc\diskio.h"
#include "sdc\ff.h"
#include "xprintf.h"

#define LED0_ON()		LED0_Write(1)
#define LED0_OFF()		LED0_Write(0)
#define LED1_ON()		LED1_Write(1)
#define LED1_OFF()		LED1_Write(0)

//----------------------------------------------------------------
// 変数定義
//----------------------------------------------------------------

reg8 SCI_STR_EXIST = 0x00;					/* 行データがある場合に"1" */
reg8 SCI_BYTES_READ = 0x00;					/* 受信データサイズ(ワーク) */
reg8 SCI_DATASIZE = 0x00;					/* 受信データサイズ */
reg8 SCI_RX_DAT[SCI_RX_SIZE] = {0x00};		/* 受信データ */
reg8 DEC_count = 0x00;						// デコード桁数
reg8 SD_DET_FLAG = 0x00;
uint16 buff;

// Real Time Clock functions
DWORD get_fattime(void);
static FRESULT scan_files(char *) CYREENTRANT;

// Work registers
FILINFO Finfo;
DWORD acc_size;			/* Work register for fs command */
WORD acc_files, acc_dirs;
	// fatfs related variables
	FATFS fatfs, *fs;
	DIR rootdir;
	FILINFO fi;
	FIL file;
	unsigned int i = 0, j = 0;
	static char buf[128];
	unsigned char b, res;
	unsigned int w1, write_bytes;
	unsigned long p2;

//----------------------------------------------------------------
// Interrupt (100mSec_Tick)

CY_ISR(isr_Tick_Timer_isr) {
//    LED1_Write(LED1_Read()?(0):(1));

// SDC マウント処理
	if(!SD_DET_Read() && !SD_DET_FLAG)			// カードがスロットに挿入され、かつ前ステップではSDマウントされていない場合
	{
			res = f_mount(0, &fatfs);			// Mount drive #0
		if(res == FR_OK)
			res = f_opendir(&rootdir, "");		// Open root directory
		if(res == FR_OK)
		{
			fs = &fatfs;
			xputs("SDC_Mounted\n");
			SD_DET_FLAG = 1;					// Mounted
			LED0_ON();
		}
	} else {
		if(SD_DET_Read() && SD_DET_FLAG)		// カードがスロットから抜去され、かつ前ステップでSDマウントされている場合
		{
			f_close(&file);
			res = f_mount(0, NULL);
			if(res == FR_OK)
			{
				xputs("SDC_Unmounted\n");
				SD_DET_FLAG = 0;				// UnMounted
				LED0_OFF();
			}
		}
	}
}

//----------------------------------------------------------------
void main()
{
    /* Place your initialization/startup code here (e.g. MyInst_Start()) */
UART_1_Start();
xfunc_in = UART_1_GetChar;		/* xgets入力デバイス指定 */
xfunc_out = UART_1_PutChar;		/* xprintf出力デバイス指定 */

// Initialize SPI (used to communicate with SD card) running at 1 MHz
SD_SPIM_Init();
SD_SPIM_Enable();
SD_SPIM_EnableInt();
SD_SPIM_Start();
isr_Tick_Timer_StartEx(isr_Tick_Timer_isr);

CyGlobalIntEnable; /* Enable all interrupts by the processor. */

DEF_Check();
SD_Access_Write(0);

//CyDelay(2000u);
//CyDelayUs(90);

	for(;;) {

		sci_recvln();								// データ受信

#ifdef	_DEBUG
		xprintf("DataSize = %u\n",SCI_DATASIZE);	// データ数出力
#endif

		if(SCI_DATASIZE==0){
			xputs("ERROR\n");						// データ無し
		} else
		{
			switch(SCI_RX_DAT[0])
			{
				case 'm':							// SDカードマウント
				case 'M':
					if(!SD_DET_Read()){
#ifdef	_DEBUG
						xputs("SDC_DET\n");
#endif
													// SDカード情報取得
							if(disk_ioctl(0, GET_SECTOR_COUNT, &p2) == RES_OK)
							{
								xprintf("Drive: %lu scts\r\n", p2);
							}
							if(disk_ioctl(0, GET_SECTOR_SIZE, &w1) == RES_OK)
							{
								xprintf("Sector size: %u\r\n", w1);
							}
							if(disk_ioctl(0, GET_BLOCK_SIZE, &p2) == RES_OK)
							{
								xprintf("Erase block: %lu sectors\r\n", p2);
							}
							if(disk_ioctl(0, MMC_GET_TYPE, &b) == RES_OK)
							{
								xprintf("MMC/SDC type: %u\r\n", (unsigned int)b);
							}

//							fs = &fatfs;
								acc_files = 0;
								acc_size = 0;
								acc_dirs = 0;
								res = scan_files("");						// Scan files stored at the root directory in the SD card
							xprintf("%u files, %lu bytes.\r\n%u folders.\r\n",
									acc_files, acc_size, acc_dirs);
							res = f_stat("test01.txt", &fi);				// Get status of the file
							if(res == FR_OK)
							{
								xprintf("%s\r\n", fi.fname);
							}

							file.fs = &fatfs;
							res = f_open(&file, "test01.txt", FA_READ);		// Open file

							if(res == FR_OK)
							{
								xprintf("%lu bytes\r\n", file.fsize);
								while(1)
								{
									f_read(&file, (char *)buf, 120, &i);
									for(j = 0; j < i; j++)
										xprintf("%c", (unsigned int)buf[j]);
									if(i < 120) break;
								}
								xprintf("\r\nEnd of File\r\n");
								f_close(&file);
							}
							else
							{
								xprintf("Failed to Open\r\n");
								f_close(&file);
							}

					} else {
						xputs("No_SDC\n");
					}
				break;

				case 'a':							// SDカードテストアクセス(既存ファイルへデータ追記)
				case 'A':
					if(!SD_DET_Read()){
#ifdef	_DEBUG
						xputs("SDC_DET\n");
#endif
						res = f_stat("test01.txt", &fi);		// Get status of the file
						if(res == FR_OK)
						{
							xprintf("%s\r\n", fi.fname);
						}
						
						res = f_open(&file, "test01.txt", FA_OPEN_ALWAYS | FA_WRITE);		// Open file
						if(res == FR_OK)
						{
							res = f_lseek(&file, f_size(&file));
							if(res == FR_OK){
								write_bytes = f_putc('C', &file);
								f_sync(&file);
									xprintf("write:%dBytes\n", write_bytes);

								write_bytes = f_printf(&file, "%s", "fatfs_WriteTest\n");
								f_sync(&file);
									xprintf("write:%dBytes\n", write_bytes);

								write_bytes = f_puts("Test_Write\n", &file);
								f_sync(&file);
									xprintf("write:%dBytes\n", write_bytes);
							}
							res = f_close(&file);
							if(res == FR_DISK_ERR) xprintf("FR_DISK_ERR\n");
						} else {
							xprintf("Failed to Open\n");
						}

					} else {
						xputs("No_SDC\n");
					}
					break;

				case 'r':							// カード挿入状況チェック
				case 'R':

					if(!SD_DET_Read()){
						xputs("SDC_Mounted\n");
					} else {
						xputs("No_SDC\n");
					}
					break;

				default:
					xputs("ERROR\n");
					break;
			}
		}
		SCI_STR_EXIST = 0;							// データありますフラグを下ろす

	}
}

//----------------------------------------------------------------
// SCI_Rx 行取得
//　やりたい：行頭～改行(CR検出)までの文字列を配列に詰めて返す。
//　　　　　　BSで1文字削除 は実装したい。
//　　　　　　改行を認識した時点でリターンコード"1"を返す。
//----------------------------------------------------------------
void sci_recvln(void)
{
	while(SCI_STR_EXIST == 0)						// 行完成まで繰り返す
	{
		uint8 d;
		for(d=0;d==0;){
			d = UART_1_GetChar();
		}
		if(d == '\r'){								// 改行(CR)が来た
			SCI_RX_DAT[SCI_BYTES_READ] = '\0';		// Nullをエンドコードとして書く。
			SCI_DATASIZE = SCI_BYTES_READ;
			SCI_BYTES_READ = 0;						// 次回受信に備え受信データサイズをクリア
			SCI_STR_EXIST = 1;						// データありますフラグを立てる
			xputs("\n");							// エコーバック(改行)
		}
		else if(d == '\b'){							// バックスペース(BS)が来た
			if(SCI_BYTES_READ > 0){					// バッファにデータが有れば
				xputs("\b \b");						// 画面上でもデータ消去
				SCI_BYTES_READ--;					// バッファのデータも1つ減らす
			}
		}
		else if(SCI_BYTES_READ >= SCI_RX_SIZE){		// 受信サイズがバッファサイズを超える
			xputs("Size Over\n");					// データ来すぎ！
			SCI_BYTES_READ = 0;						// 次回受信に備え受信データサイズをクリア
		}
		else if(d >= 0x20 && d <= 0x7E){			// 文字(英数字)が来た
			SCI_RX_DAT[SCI_BYTES_READ] = d;			// そのままデータとして積む。
			SCI_BYTES_READ++;
			xprintf("%c", d);						// echo back
		}
	}
	return;
}

//----------------------------------------------------------------
// ASCII配列データ→int変数化
//　やりたい：UARTから得たコマンド列より、引数で指定した開始バイトから
//　　　　　　終了文字(\r or \0 or space)までの値を数値として返す。
//　　　　　　得たデータは、戻り値として返す。
//----------------------------------------------------------------
uint16 str2dec(uint8 start_byte)
{
	uint8	i, j, wk;
	uint16	dec_num = 0;
	uint8	end_byte = 0;
	char8	num_dat[str2dec_buf_size] = {0x00};

	for (i=start_byte;i<=str2dec_buf_size;i++)
	{
		if(SCI_RX_DAT[i] == '\r' || SCI_RX_DAT[i] == '\0' || SCI_RX_DAT[i] == ' ')
		{
			end_byte = i - 1;
			break;
		}
	}

	if(end_byte == start_byte){
		dec_num=0;							// 固定値0として処理
		return 0;							// いきなり終わった
	}
	DEC_count = i - start_byte;				// それ以外の時は何文字来たかカウント

	if(end_byte < DEC_count)
	{
		wk = 0;
	} else
	{
		wk = (end_byte - DEC_count);
	}

	if (wk == 0)
	{
		for(i=0;i<=end_byte;i++)
		{
			num_dat[i] = SCI_RX_DAT[i];
		}
	} else
	{
		j = 0;
		for(i=wk;i<=end_byte;i++)
		{
			num_dat[j] = SCI_RX_DAT[i+1];
			j++;
		}
	}


	dec_num = atoi(num_dat);
	return dec_num;
}

void DEF_Check(void)
{
	if (SCI_RX_SIZE > str2dec_buf_size)
	{
		xputs("[ ERR ] str2dec_buf_size too small.\n");
	}
}

/*---------------------------------------------------------*/
/* User Provided RTC Function for FatFs module             */
/*---------------------------------------------------------*/
/* This is a real time clock service to be called from     */
/* FatFs module. Any valid time must be returned even if   */
/* the system does not support an RTC.                     */
/* This function is not required in read-only cfg.         */

DWORD get_fattime(void)
{
	DWORD tmr;


//	_DI();
	/* Pack date and time into a DWORD variable */
	tmr =	  (((DWORD)2010 - 1980) << 25)
			| ((DWORD)11 << 21)
			| ((DWORD)1 << 16)
			| (WORD)(0 << 11)
			| (WORD)(0 << 5)
			| (WORD)(0 >> 1);
//	_EI();

	return tmr;
}

static FRESULT scan_files(
	char* path		/* Pointer to the path name working buffer */
) CYREENTRANT
{
	DIR dirs;
	FRESULT res;
	BYTE i;
	char *fn;

	if ((res = f_opendir(&dirs, path)) == FR_OK) {
		i = strlen(path);
		while (((res = f_readdir(&dirs, &Finfo)) == FR_OK) && Finfo.fname[0]) {
			if (_FS_RPATH && Finfo.fname[0] == '.') continue;
#if _USE_LFN
			fn = *Finfo.lfname ? Finfo.lfname : Finfo.fname;
#else
			fn = Finfo.fname;
#endif
			if (Finfo.fattrib & AM_DIR) {
				acc_dirs++;
				*(path+i) = '/'; strcpy(path+i+1, fn);
//				res = scan_files(path);
				*(path+i) = '\0';
				if (res != FR_OK) break;
			} else {
/*				xprintf("%s/%s\n", path, fn);	*/
				acc_files++;
				acc_size += Finfo.fsize;
			}
		}
	}

	return res;
}

/* [] END OF FILE */
